local L = LibStub("AceLocale-3.0"):GetLocale("Click2Cast", true)
local Click2Cast = LibStub("AceAddon-3.0"):GetAddon("Click2Cast")

local actions = {
	menu = L["Menu"],
	spell = L["Spell"],
	target = L["Target"],
	focus = L["Focus"],
	assist = L["Assist"],
	macro = L["Macro"],
	macroname = L["Macro Name"],
	none = L["None"],
}

local actionstring = {
	spell = L["Spell Name"],
	macro = L["Macro Text"],
	macroID = L["Macro ID"],
	macroname = L["Macro Name"],
}

local actiondesc = {
	spell = L["Enter the name of the spell to cast."],
	macro = L["Enter the macro code to execute."],
	macroname = L["Enter the name of the macro to execute. Useful for multi-line macros."],
	macroID = L["Enter the ID of the macro to execute. Useful for multi-line macros."],
}

local kbactions = {
	spell = L["Spell"],
	macroID = L["Macro"],
	none = L["None"],
}

local translatedModifiers = {
	SHIFT = L["Shift"],
	ALT = L["Alt"],
	CTRL = L["Ctrl"],
}

local niceModifier = setmetatable({}, {__index =
	function(self, key)
		if type(key) ~= "string" then return nil end
		local x = strsplit("-", key)
		local t = {}
		for i = 1, select("#", x) do
			local cur = (select(i, x))
			if not translatedModifiers[cur] then break end
			t[#t + 1] = translatedModifiers[cur]
		end
		if #t < 1 then return nil end
		self[key] = table.concat(t, "-")
		for i in ipairs(t) do t[i] = nil end
		return self[key]
	end
})

local keybindingOnly = {BUTTON1=true, BUTTON2=true, BUTTON3=true, BUTTON4=true, BUTTON5=true}

local bindCreateMouse = {
	button = "BUTTON1",
	modifier1 = "1",
	modifier2 = "1",
}

local bindCreateKB = {
	button = "",
	modifier1 = "1",
	modifier2 = "1",
}

local options = {
	type = "group",
	args = {
		help = {
			type = "description",
			name = L["Click2Cast allows you to bind spells, macros and other actions to mouse clicks and keyboard strokes. For example this allows you to heal and buff your party and raid members easily just by clicking on them."],
		}
	}
}

local mouseOptions = {
	name = L["Mouse Bindings"],
	type = "group",
	hidden = false,	
	get = function(info)
		return bindCreateMouse[info[#info]]
	end,
	set = function(info, value)
		bindCreateMouse[info[#info]] = value
	end,
	childGroups = "tree",
	args = {
		group1 = {
			name = "",
			type = "group",
			inline = true,
			order = 1,
			args = {
				button = {
					name = L["Button"],
					type = "select",
					desc = L["Select binding button."],
					values = {BUTTON1 = L["Button 1"], BUTTON2 = L["Button 2"], BUTTON3 = L["Button 3"], BUTTON4 = L["Button 4"], BUTTON5 = L["Button 5"]},
					order = 1,
					width = "half",
				},
				modifier1 = {
					name = L["Modifier 1"],
					type = "select",
					desc = L["Select modifier button."],
					values = {["1"] = L["None"], SHIFT = L["Shift"], ALT = L["Alt"], CTRL = L["Ctrl"]},
					order = 2,
					width = "half",
				},
				modifier2 = {
					name = L["Modifier 2"],
					type = "select",
					desc = L["Select modifier button."],
					values = function() 
						local t = {["1"] = L["None"], SHIFT = L["Shift"], ALT = L["Alt"], CTRL = L["Ctrl"]}
						if bindCreateMouse.modifier1 ~= "1" then
							t[bindCreateMouse.modifier1] = nil
						end
						return t
					end,
					order = 3,
					width = "half",
					hidden = function() return bindCreateMouse.modifier1 == "1" end
				},
			},
		},
		group2 = {
			name = "",
			type = "group",
			inline = true,
			order = 2,
			args = {
				create = {
					name = L["Create New Binding"],
					type = "execute",
					desc = L["Creates a configurable click binding."],
					func = function() Click2Cast:CreateSubmenu("mouse") end,
				},
			}
		}
	}
}

local kbOptions = {
	name = L["Keyboard Bindings"],
	type = "group",
	hidden = false,	
	get = function(info)
		return bindCreateKB[info[#info]]
	end,
	set = function(info, value)
		bindCreateKB[info[#info]] = value
	end,
	childGroups = "tree",
	args = {
		group1 = {
			name = "",
			type = "group",
			inline = true,
			order = 1,
			args = {
				button = {
					name = L["Button"],
					type = "input",
					desc = L["Select binding button."],
					order = 1,
					width = "half",
					set = function(info, value)
						bindCreateKB[info[#info]] = string.match(value, "%S") or ""
					end,
				},
				modifier1 = {
					name = L["Modifier 1"],
					type = "select",
					desc = L["Select modifier button."],
					values = {["1"] = L["None"], SHIFT = L["Shift"], ALT = L["Alt"], CTRL = L["Ctrl"]},
					order = 2,
					width = "half",
				},
				modifier2 = {
					name = L["Modifier 2"],
					type = "select",
					desc = L["Select modifier button."],
					values = function() 
						local t = {["1"] = L["None"], SHIFT = L["Shift"], ALT = L["Alt"], CTRL = L["Ctrl"]}
						if bindCreateKB.modifier1 ~= "1" then
							t[bindCreateKB.modifier1] = nil
						end
						return t
					end,
					order = 3,
					width = "half",
					hidden = function() return bindCreateKB.modifier1 == "1" end
				},
			}
		},
		group2 = {
			name = "",
			type = "group",
			inline = true,
			order = 2,
			args = {
				create = {
					name = L["Create New Binding"],
					type = "execute",
					desc = L["Creates a configurable click binding."],
					func = function() Click2Cast:CreateSubmenu("kb") end,
					disabled = function()
						if string.match(bindCreateKB.button, "%a(%a+)") or bindCreateKB.button == "" then
							return true
						end
					end
				},	
			}
		}			
	}
}

local function getBindingLocale(text)
	if text == nil or text == "NONE" then
		return nil
	end
	text = tostring(text):upper()
	local shift, ctrl, alt
	local modifier
	while true do
		if text == "-" then
			break
		end
		modifier, text = strsplit('-', text, 2)
		if text then
			if modifier ~= "SHIFT" and modifier ~= "CTRL" and modifier ~= "ALT" then
				return false
			end
			if modifier == "SHIFT" then
				if shift then
					return false
				end
				shift = true
			end
			if modifier == "CTRL" then
				if ctrl then
					return false
				end
				ctrl = true
			end
			if modifier == "ALT" then
				if alt then
					return false
				end
				alt = true
			end
		else
			text = modifier
			break
		end
	end
	local s
	if text == "BUTTON1" then
		s = L["Button 1"]
	elseif text == "BUTTON2" then
		s = L["Button 2"]
	elseif text == "BUTTON3" then
		s = L["Button 3"]
	elseif text == "BUTTON4" then
		s = L["Button 4"]
	elseif text == "BUTTON5" then
		s = L["Button 5"]
	else
		s = GetBindingText(text, "KEY_") or text
	end
	if shift then
		s = "Shift-" .. s
	end
	if ctrl then
		s = "Ctrl-" .. s
	end
	if alt then
		s = "Alt-" .. s
	end
	return s
end

local function getOrder(binding)
	local num = 0
	if binding:find("ALT%-") then
		num = num + 1
	end
	if binding:find("CTRL%-") then
		num = num + 2
	end
	if binding:find("SHIFT%-") then
		num = num + 3
	end

	local button = tonumber(binding:match("BUTTON(%d)$"))
	if binding:find("BUTTON") then
		num = num + button * 8
	end
	return num
end

local bindOptions = {
	name = function(info) return getBindingLocale(info[#info]) end,
	order = function(info) return getOrder(info[#info]) end,
	type = "group",
	hidden = false,
	get = function(info) return Click2Cast.db.char.clickOptions[info[#info-1]][info[#info]] end,
	set = function(info, value) Click2Cast.db.char.clickOptions[info[#info-1]][info[#info]] = value Click2Cast:SetAllFrames() Click2Cast:SetKeyboardBindings() end,
	args = {
		action = {
			name = L["Action"],
			type = "select",
			desc = L["Select binding action."],
			values = function(info)
				if info.appName == "Click2CastMouse" then
					return actions
				else
					return kbactions
				end
			end,
			order = 1,
			set = function(info, value)
				Click2Cast.db.char.clickOptions[info[#info-1]][info[#info]] = value
				Click2Cast.db.char.clickOptions[info[#info-1]].actiontext = nil
				Click2Cast:SetAllFrames() 
				Click2Cast:SetKeyboardBindings() 
			end,
		},
		actiontext = {
			name = function(info)
				local action = Click2Cast.db.char.clickOptions[info[#info-1]].action
				return actionstring[action]
			end,
			desc = function(info)
				local action = Click2Cast.db.char.clickOptions[info[#info-1]].action
				return actiondesc[action]
			end,
			type = "input",
			order = 2,
			hidden = function(info)
				local action = Click2Cast.db.char.clickOptions[info[#info-1]].action
				if not (Click2Cast.actionText[action]) then
					return true
				end
			end
		},
		rank = {
			name = L["Spell Rank"],
			type = "input",
			desc = L["Set spell rank."],
			order = 3,
			hidden = function(info)
				local action = Click2Cast.db.char.clickOptions[info[#info-1]].action
				if action ~= "spell" then
					return true
				end
			end,
			set = function(info, value)
				Click2Cast.db.char.clickOptions[info[#info-1]][info[#info]] = value:match("(%d+)")
				self:SetAllFrames()
				self:SetKeyboardBindings()
			end
		},
		outside = {
			name = L["Outside Unitframes"],
			type = "toggle",
			desc = L["Allows you to use the mouse click binding everywhere, not just on unit frames."],
			order = 4,
			hidden = function(info)
				if info.appName == "Click2CastKB" then
					return true
				end
				local action = Click2Cast.db.char.clickOptions[info[#info-1]].action
				if action ~= "spell" then
					return true
				end
			end,
		},		
	}
}

function Click2Cast:CreateMenu()
	LibStub("AceConfig-3.0"):RegisterOptionsTable("Click2Cast", options)
	Click2Cast.menu = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Click2Cast", "Click2Cast")
	
	LibStub("AceConfig-3.0"):RegisterOptionsTable("Click2CastMouse", mouseOptions)
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Click2CastMouse", L["Mouse Bindings"], "Click2Cast")

	LibStub("AceConfig-3.0"):RegisterOptionsTable("Click2CastKB", kbOptions)
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Click2CastKB", L["Keyboard Bindings"], "Click2Cast")	
	self:UpdateMenu()
end

function Click2Cast:CreateSubmenu(input)
	local tbl, options, k
	if input == "mouse" then
		tbl = bindCreateMouse
		options = mouseOptions
	else
		tbl = bindCreateKB
		options = kbOptions
	end
	if tbl.modifier1 ~= "1" and tbl.modifier2 ~= "1" then
		k = string.format("%s-%s-%s", tbl.modifier1, tbl.modifier2, tbl.button)
	elseif tbl.modifier1 ~= "1" then
		k = string.format("%s-%s", tbl.modifier1, tbl.button)
	else
		k = tbl.button
	end
	if not options.args[k] then
		options.args[k] = bindOptions
	end
end

function Click2Cast:UpdateMenu()
	for k, v in pairs(self.db.char.clickOptions) do
		if v.action ~= "none" then
			if string.find(k,"BUTTON%d") then
				mouseOptions.args[k] = bindOptions
			else
				kbOptions.args[k] = bindOptions
			end
		end
	end
end

--[[
local function getOrder(binding)
	local num = 0
	if binding:find("ALT%-") then
		num = num + 3
	end
	if binding:find("CTRL%-") then
		num = num + 4
	end
	if binding:find("SHIFT%-") then
		num = num + 5
	end

	local button = tonumber(binding:match("BUTTON(%d)$"))
	if binding:find("BUTTON") then
		num = num + button * 8
	else
		num = 1
	end
	return num
end


local function createSubMenu(self, value)
	local name = dewdrop:EncodeKeybinding(value)
	local validateActions
	if value:find("BUTTON") then
		validateActions = actions
	else
		validateActions = kbactions
	end
	self.optionsTable.args[value] = {
		name = name or value.."bug",
		desc = name or value.."bug",
		order = getOrder(value),
		type = "group",
		args = {
			actions = {
				type = "text",
				name = L["Set Action"],
				desc = L["actions"],
				order = 1,
				validate = validateActions,
				get = function()
					return self.db.char.clickOptions[value].action
				end,
				set = function(option)
					self.db.char.clickOptions[value].action = option
					self:SetAllFrames()
					self:SetKeyboardBindings()
					if self.actionText[self.db.char.clickOptions[value].action] then
						self.optionsTable.args[value].args.action.name = L[option]
						self.optionsTable.args[value].args.action.desc = L[option.."Desc"]
					end
				end,
			},
			remove = {
				type = "execute",
				name = L["Remove"],
				desc = L["RemoveDesc"],
				func = function()
					self.db.char.clickOptions[value] = nil
					self.optionsTable.args[value] = nil
					self:DeleteAction(value)
					self:DisableKeyboardBindings(value)
				end,
			},
			action = {
				name = L["Action Text"],
				type = "text",
				desc = L["actiontext"],
				order = 2,
				get = function()
					return self.db.char.clickOptions[value].actiontext or ""
				end,
				usage = "",
				set = function(option)
					self.db.char.clickOptions[value].actiontext = option
					self:SetAllFrames()
					self:SetKeyboardBindings()
				end,
				hidden = function()
					return not (self.actionText[self.db.char.clickOptions[value].action])
				end,
			},
			rank = {
				name = L["Rank"],
				type = "text",
				desc = L["rankDesc"],
				order = 3,
				get = function()
					return self.db.char.clickOptions[value].rank or ""
				end,
				usage = "",
				set = function(option)
					self.db.char.clickOptions[value].rank = option:match("(%d+)")
					self:SetAllFrames()
					self:SetKeyboardBindings()
				end,
				hidden = function()
					return not (self.db.char.clickOptions[value].action == "spell")
				end,
			},
			withoutframes = {
				name = L["Outside Unitframes"],
				type = "toggle",
				desc = L["Mousebind works outside unit frames"],
				get = function()
					return self.db.char.clickOptions[value].outside
				end,
				set = function(option)
					self.db.char.clickOptions[value].outside = option
					self:SetAllFrames()
					self:SetKeyboardBindings()
				end,
				hidden = function()
					return not (self.db.char.clickOptions[value].action == "spell") or not value:find("BUTTON")
				end,
				order = 4,
			},
		}
	}
	if self.actionText[self.db.char.clickOptions[value].action] then
		self.optionsTable.args[value].args.action.name = L[self.db.char.clickOptions[value].action]
		self.optionsTable.args[value].args.action.desc = L[self.db.char.clickOptions[value].action.."Desc"]
	end
end

function Click2Cast:CreateMenu()
	self.optionsTable = {
		name = L["C2C"],
		desc = L["C2CDesc"],
		order = 3,
		handler = Click2Cast,
		type = "group",
		args = {},
		disabled = function()
			if InCombatLockdown() then
				return true
			end
		end,
	}
	self.optionsTable.args.new = {
		name = L["New Keybinding"],
		desc = L["New Keybinding for a action"],
		order = 200,
		type = "text",
		validate = "keybinding",
		set = function(value)
			if value and not self.optionsTable.args[value] then
				createSubMenu(self, value)
			end
		end,
		get = function()
			return nil
		end,
--		keybindingOnly = keybindingOnly,
	}
	for k in pairs(self.db.char.clickOptions) do
		createSubMenu(self, k)
	end

	Click2Cast:RegisterChatCommand("/Click2Cast", "/c2c", "/cc", self.optionsTable, "CLICK2CAST")
	Click2Cast.OnMenuRequest = self.optionsTable
end

local tmp = {}
local function mysort(alpha, bravo)
	if not alpha or not bravo then
		return false
	end
	return getOrder(alpha) < getOrder(bravo)
end

function Click2Cast:OnTooltipUpdate()
	GameTooltip:AddLine("Click2Cast")

	local db = self.db.char.clickOptions
	for k,v in pairs(db) do
		table.insert(tmp, k)
	end
	table.sort(tmp, mysort)
	local last_button = nil
	for i,k in ipairs(tmp) do
		if k:find("BUTTON") then
			tmp[i] = nil
			local v = db[k]
			local button = tonumber(k:match("BUTTON(%d)$"))
			if last_button ~= button then
				last_button = button
				GameTooltip:AddLine(" ")
				GameTooltip:AddLine(dewdrop:EncodeKeybinding("BUTTON" .. button), 1,1,1)
			end
			local text
			if not k:find("%-") then
				text = L["No Modifier"]
			else
				text = niceModifier[k]
			end
			if v.actiontext and v.action == "spell" then
				GameTooltip:AddDoubleLine(text, v.actiontext, 1,1,0, 1,1,1)
			else
				GameTooltip:AddDoubleLine(text, L[v.action], 1,1,0, 1,1,1)
			end
		end
	end

	GameTooltip:AddLine(" ")
	GameTooltip:AddLine(L["|cffeda55fRight-Click|r to configure clickcast actions."], 0.2, 1, 0.2, 1)
end

function Click2Cast:RecreateMenu()
	for k in pairs(self.optionsTable.args) do
		if k:find("^ALT.-BUTTON%d$") or k:find("^CTRL.-BUTTON%d$") or k:find("^SHIFT.-BUTTON%d$") or k:find("^BUTTON%d$") then
			self.optionsTable.args[k] = nil
		end
	end
	for k in pairs(self.db.char.clickOptions) do
		createSubMenu(self, k)
	end
end
]]
